package com.ruoyi.mcu.server;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ArrayUtil;
import com.ruoyi.advertiser.service.IAdHandleService;
import com.ruoyi.common.utils.sign.Base64;
import com.ruoyi.mcu.config.McuSocketConfig;
import com.ruoyi.mcu.constant.McuBeatIdentificationConstant;
import com.ruoyi.mcu.constant.McuBeatPackTypeConstant;
import com.ruoyi.mcu.domain.AdHeartBeatPackage;
import com.ruoyi.mcu.util.AdParseUtil;
import com.ruoyi.mcu.util.CRC16ModbusUtil;
import com.ruoyi.mcu.util.DataTransitionUtil;
import lombok.extern.slf4j.Slf4j;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.SocketChannel;
import java.util.Locale;
import java.util.Queue;
import java.util.concurrent.ArrayBlockingQueue;

/**
 * @author wzsdch_p751
 * @date 2022/07/18 13:11
 * 该对象代表一个正在进行的tcp连接
 **/
@Slf4j(topic = "sys-mcu")
public class DealChannel {

    private final McuSocketConfig mcuSocketConfig;

    private Selector selector;

    private final DealServer dealServer;

    private SocketChannel channel;

    private Long lastReadDate;

    private String imei;

    private final Queue<ByteBuffer> writeQueue = new ArrayBlockingQueue<ByteBuffer>(16);

    public Selector getSelector() {
        return selector;
    }

    public void setSelector() {
        this.selector = selector;
    }

    public SocketChannel getChannel() {
        return channel;
    }

    //    public DealChannel(SocketChannel channel){
//        this.channel = channel;
//    }
    public DealChannel(McuSocketConfig mcuSocketConfig,
                       IAdHandleService adHandleService,
                       DealServer dealServer
                       ){
        this.mcuSocketConfig = mcuSocketConfig;
        this.dealServer = dealServer;
    }
    public DealChannel init(SocketChannel channel,Selector selector){
        this.channel = channel;
        this.selector = selector;
        this.setLastReadDate(System.currentTimeMillis());
        return this;
    }

    public String getImei() {
        return imei;
    }

    public void setImei(String imei) {
        this.imei = imei;
    }

    public void read(SelectionKey selectionKey, String remoteAddress) throws IOException {
        SocketChannel socketChannel = (SocketChannel) selectionKey.channel();
        String content = "";
        String base64 = "";

        byte[] responseBytes = null;
        // 返回该SelectionKey对应的 Channel，其中有数据需要读取,则读取它送过来的数据
        // 获取数据
        ByteBuffer buffer = ByteBuffer.allocate(mcuSocketConfig.getBufferSize());

        // 读取数据到缓冲区buff
        int readNum ;
        while ((readNum=socketChannel.read(buffer))>0) {
            socketChannel.read(buffer);
            buffer.flip();
        }
        // 客户如果正常断开连接 read()方法的返回值是 -1
        if (readNum == -1) {
            log.info("时间: {} ，设备IP：{} 断开连接 <br/>", DateUtil.now(),remoteAddress);
            selectionKey.cancel();
            socketChannel.close();
            return;
        }

        //此处是把收到的信息 buff--》byte[]（类似于 “01 02 03 04 05 06”）--》16进制数据
        content = Convert.toHex(ArrayUtil.toArray(buffer)).toUpperCase(Locale.ROOT);
        base64 = Base64.encode(ArrayUtil.toArray(buffer));

        //log.info("接收到数据：" + content + "\r\n" + DateUtil.now() + "长度：" + content.length()+"\r\n"+"base64：" + base64);
        //log.info("---------------------------------------------------------------------------------");
        log.info("接收到数据：时间:{}   长度：{}\r\nHEX内容：{}\r\nBASE64内容：{}",DateUtil.now(),content.length(),content,base64);

        AdHeartBeatPackage beatPack = AdParseUtil.getHeartBeatPackage(buffer.array());
        if (beatPack == null) {
            log.info("心跳包对象数据为null。");
            return;
        }
        if ((beatPack.getIdentification() & 0x01) == 0x01) {
            String hex = Convert.toHex(DataTransitionUtil.intsToBytes(beatPack.getPacketData()));
            log.info("数据SM4解密后：{}", hex.toUpperCase());
        }

        //获取校验结果
        boolean checkResult = CRC16ModbusUtil.getCheckResult(beatPack.getPacketData(), beatPack.getCrc16());
        // crc16校验
        if (!checkResult) {
            log.error("CRC16校验失败！");
            return;
        }
        //log.info("心跳包数据校验成功！");

        String imei = beatPack.getImeiStr();
        this.setImei(imei);
        log.info("读取到的设备IMEI信息：**********" + imei + "*************<br/>");


        // 如果结果不是0x80则不需要回复心跳数据
        if ((beatPack.getIdentification() & McuBeatIdentificationConstant.BEAT_IDENTIFICATION_0X80) != McuBeatIdentificationConstant.BEAT_IDENTIFICATION_0X80) {
            return;
        }

        // 上行心跳包 0x01
        if (beatPack.getPackageType() == McuBeatPackTypeConstant.UP_BEAT_PACKAGE_TYPE) {
            // 处理心跳包
            responseBytes = dealServer.upBeatReply(beatPack);
        }

        // 图片文件传输包 0x05
        if (beatPack.getPackageType() == McuBeatPackTypeConstant.IMAGE_TRANSMIT_PACKAGE_TYPE) {
            // 图片下载&&图片文件传输状态
            responseBytes = dealServer.imageReply(beatPack);
        }

        // 固件升级 升级包 0x04
        if (beatPack.getPackageType() == McuBeatPackTypeConstant.UPGRADE_PACKAGE_TYPE) {
            //处理 固件包下载 && 固件升级状态
            responseBytes = dealServer.upgradeReply(beatPack);
        }

        // SE平台业务  0x06 激活，签名
        if (beatPack.getPackageType() == McuBeatPackTypeConstant.SE_PACKAGE_TYPE) {
            responseBytes = dealServer.seBeatReply(beatPack);
        }

        //回复消息
        if (responseBytes != null) {
            ByteBuffer writebuffer = ByteBuffer.wrap(responseBytes);
            writeQueue.add(writebuffer);
            selectionKey.interestOps(SelectionKey.OP_WRITE);
        }
    }
    public void write(SelectionKey selectionKey) throws IOException {
        ByteBuffer buffer;
        while ((buffer=writeQueue.poll())!=null){
            channel.write(buffer);
        }
        selectionKey.interestOps(SelectionKey.OP_READ);
    }


    public Long getLastReadDate() {
        return lastReadDate;
    }

    public void setLastReadDate(Long lastReadDate) {
        this.lastReadDate = lastReadDate;
    }
}
