package com.ruoyi.mcu.util;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.HexUtil;
import com.ruoyi.mcu.config.McuSm4KeyConfig;
import com.ruoyi.mcu.constant.McuActivationConstant;
import com.ruoyi.mcu.constant.McuBeatPackHeadAndTailConstant;
import com.ruoyi.mcu.domain.AdHeartBeatPackage;
import com.ruoyi.mcu.domain.AdMessageBody;
import com.ruoyi.mcu.domain.TlvFrame;
import com.ruoyi.setsm.domain.SeActivating;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import static com.ruoyi.mcu.constant.McuBeatPackTagConstant.*;

/**
 * @author zzw
 * @date 2021/6/8
 * @Description: 解析数据工具类
 */
@Component
@Slf4j
public class AdParseUtil {

    /**
     * 解析包数据
     *
     * @param beatDataPre 处理前的包数据
     * @return AdHeartBeatPackage
     */
    public static AdHeartBeatPackage getHeartBeatPackage(byte[] beatDataPre) {

        // 处理包数据变为int数组，如果小于0 加256
        int[] beatData = new int[beatDataPre.length];
        for (int i = 0; i < beatDataPre.length; i++) {
            beatData[i] = beatDataPre[i];
            if (beatData[i] < 0) {
                beatData[i] += 256;
            }
        }
        AdHeartBeatPackage beatPackage = new AdHeartBeatPackage();
        int i = 0; // beatData数组下标
        // 如果包数据长度小于临界值 返回空
        if (beatData.length < 3) {
            return null;
        }
        // 包头
        beatPackage.setPackageHead((byte) (beatData[i++]));
        // 校验包头
        if (beatPackage.getPackageHead() != McuBeatPackHeadAndTailConstant.PACKAGE_HEAD) {
            return null;
        }
        // 得到IMEI
        int[] devIMEI = new int[8];
        System.arraycopy(beatData, 1, devIMEI, 0, 8);

        beatPackage.setIMEI(devIMEI);
        i += 8;
        // 协议版本
        beatPackage.setProtocolVersion(beatData[i++]);
        // 设备类型
        int[] equipmentType = new int[2];
        System.arraycopy(beatData, i, equipmentType, 0, equipmentType.length);
        beatPackage.setEquipmentType(equipmentType);
        i += 2;
        // 包序号
        beatPackage.setPackageSeq((beatData[i++] << 8) | beatData[i++]);
        // 标识
        beatPackage.setIdentification((byte) beatData[i++]);
        // 包类型
        beatPackage.setPackageType((byte) (beatData[i++]));
        // 标签数量
        beatPackage.setLabelNum(Convert.toInt(beatData[i++]));
        // 消息体总长度
        beatPackage.setMsgLength((beatData[i++] << 8) | beatData[i++]);
        if (beatPackage.getMsgLength() > (1000 - 22)) {
            return null;
        }
        // 消息体
        int[] packetData = new int[beatPackage.getMsgLength()];
        System.arraycopy(beatData, i, packetData, 0, beatPackage.getMsgLength());

        // 解密操作
        if ((beatPackage.getIdentification() & 0x01) == 0x01) {
            packetData = DataTransitionUtil.bytesToInts(SM4Util.payloadEncrypt(DataTransitionUtil.intsToBytes(packetData), Convert.hexToBytes(McuSm4KeyConfig.SM4KEY), DataTransitionUtil.intsToBytes(beatPackage.getIMEI()), beatPackage.getPackageSeq()));
            String hex = Convert.toHex(DataTransitionUtil.intsToBytes(packetData));
            log.info("消息体SM4解密后：{}", hex);
            System.arraycopy(packetData, 0, beatData, i, packetData.length);
        }
        // 消息体数组 (补充说明：并不是单独的消息体，是从包头开始到消息体的数据，用于crc16校验)
        int[] tempBeatData = new int[i + packetData.length];
        System.arraycopy(beatData, 0, tempBeatData, 0, i + packetData.length);
        beatPackage.setPacketData(tempBeatData);
        // 获得消息体对象
        beatPackage.setDeviceInfo(getMsgBody(packetData));

        // crc16
        beatPackage.setCrc16((long) (((beatData[19 + beatPackage.getMsgLength()] < 0
                ? beatData[19 + beatPackage.getMsgLength()] + 256
                : beatData[19 + beatPackage.getMsgLength()]) << 8)
                | (beatData[19 + beatPackage.getMsgLength() + 1] < 0
                ? beatData[19 + beatPackage.getMsgLength() + 1] + 256
                : beatData[19 + beatPackage.getMsgLength() + 1])));

        // 包尾
        beatPackage.setPackageTail((byte) (beatData[19 + beatPackage.getMsgLength() + 2] & 0xff));
        // 校验包尾
        if (beatPackage.getPackageTail() != McuBeatPackHeadAndTailConstant.PACKAGE_TAIL) {
            return null;
        }
        return beatPackage;
    }

    /**
     * 解析消息体
     *
     * @param beatData 消息体
     * @return 消息体对象（设备信息）
     */
    public static AdMessageBody getMsgBody(int[] beatData) {
        int readLen = 0; // 读取长度
        int tagLen = 0; // 标签长度
        int len = beatData.length - 2; // 消息总长度
        // TLV结构对象
        TlvFrame tlvFrame = new TlvFrame();
        AdMessageBody mDeviceInfoTemp = new AdMessageBody(); // 设备信息对象
        while (readLen < len) {
            tlvFrame.setTag(beatData[readLen]);
            tlvFrame.setLength((beatData[readLen + 1] << 8) | beatData[readLen + 2]);
            tagLen = readLen;
            readLen += tlvFrame.getLength() + 3;
            switch (tlvFrame.getTag()) {
                case TAG_DEV_MODLE_CSQ:
                    mDeviceInfoTemp.setCsq(beatData[tagLen + 3]);
                    break;
                case TAG_DEV_MODLE_PCI:
                    mDeviceInfoTemp.setPci((beatData[tagLen + 3] << 8) | beatData[tagLen + 4]);
                    break;
                case TAG_DEV_MODLE_ICCID:
                    int[] iccid = new int[10];
                    System.arraycopy(beatData, tagLen + 3, iccid, 0, 10);
                    mDeviceInfoTemp.setIccid(Convert.toHex(DataTransitionUtil.intsToBytes(iccid)));
                    break;
                case TAG_DEV_HW_REV:
                    mDeviceInfoTemp.setHw(beatData[tagLen + 3]);
                    break;
                case TAG_DEV_SW_REV:
                    mDeviceInfoTemp.setSw(beatData[tagLen + 3]);
                    break;
                case TAG_DEV_HBI:               //是否存储心跳时间
                    mDeviceInfoTemp.setHbiTime((beatData[tagLen + 3] << 8) | beatData[tagLen + 4]);
                    break;
                case TAG_DEV_BATTERY_LEV:
                    mDeviceInfoTemp.setPowerVol(beatData[tagLen + 3]);
                    break;
                case TAG_DEV_RESP_HBI:
                    break;
                case TAG_DEV_ERROR_CODE:
                    mDeviceInfoTemp.setErrorCode((beatData[tagLen + 3] << 8) | beatData[tagLen + 4]);
                    break;
                case TAG_DEV_DISIMAGE_CYCLE:
                    mDeviceInfoTemp.setDisplayTime((beatData[tagLen + 3] << 8) | beatData[tagLen + 4]);
                    break;
                case TAG_DEV_LORA_FRQ:
                    mDeviceInfoTemp.setLoraFrq((beatData[tagLen + 3] << 24)
                            | (beatData[tagLen + 4] << 16)
                            | (beatData[tagLen + 5] << 8)
                            + beatData[tagLen + 6]);
                    break;
                case TAG_DEV_DISKIO_SIZE:
                    mDeviceInfoTemp.setDiskIoTotalSize(((beatData[tagLen + 3] << 24) | (beatData[tagLen + 4] << 16) | (beatData[tagLen + 5] << 8) + beatData[tagLen + 6]));
                    mDeviceInfoTemp.setDiskIoUserSize(((beatData[tagLen + 7] << 24) | (beatData[tagLen + 8] << 16) | (beatData[tagLen + 9] << 8) + beatData[tagLen + 10]));
                    break;
                case TAG_DEV_SERVER_IMAGE_INFO:         //服务器图片设备绑定关系

                    break;
                case TAG_DEV_LOCAL_IMAGE_INFO:
                    mDeviceInfoTemp.setImageNums((beatData[tagLen + 3] << 8) | beatData[tagLen + 4]);
                    int[] imageRepIndex = new int[100];
                    int[] imageRepDispTime = new int[100];
                    for (int i = 0; i < mDeviceInfoTemp.getImageNums(); i++) {
                        imageRepIndex[i] = ((beatData[tagLen + 5 + 6 * i] << 24) | (beatData[tagLen + 6 + 6 * i] << 16) | (beatData[tagLen + 7 + 6 * i] << 8) + beatData[tagLen + 8 + 6 * i]);
                        imageRepDispTime[i] = (beatData[tagLen + 9 + 6 * i] << 8) | beatData[tagLen + 10 + 6 * i];
                    }
                    mDeviceInfoTemp.setImageRepIndex(imageRepIndex);
                    mDeviceInfoTemp.setImageRepDispTime(imageRepDispTime);
                    break;
                case TAG_DEV_LOCAL_TIMESTAMP:
                    mDeviceInfoTemp.setLocalTime((beatData[tagLen + 3] << 24)
                            | (beatData[tagLen + 4] << 16)
                            | (beatData[tagLen + 5] << 8)
                            + beatData[tagLen + 6]);
                    break;
                case TAG_DEV_UPGRADE_REQ:
                    int firmwareIndex = ((beatData[tagLen + 3] << 24)
                            | (beatData[tagLen + 4] << 16)
                            | (beatData[tagLen + 5] << 8)
                            + beatData[tagLen + 6]);

                    int firmwareLength = (beatData[tagLen + 7] << 8)
                            + beatData[tagLen + 8];

                    mDeviceInfoTemp.setFirmwareIndex(firmwareIndex).setFirmwareLength(firmwareLength);
                    break;
                case FIRMWARE_UPGRADE_STATUS_TAG:
                    mDeviceInfoTemp.setFirewareStatus(beatData[tagLen + 3]);
                    break;
                case TAG_DEV_IMAGE_DOWN_RESP:
                    int imgIndex = ((beatData[tagLen + 3] << 24)
                            | (beatData[tagLen + 4] << 16)
                            | (beatData[tagLen + 5] << 8)
                            + beatData[tagLen + 6]);
                    int imageSpace = (beatData[tagLen + 7] << 24)
                            | (beatData[tagLen + 8] << 16)
                            | (beatData[tagLen + 9] << 8)
                            + beatData[tagLen + 10];

                    int imageLen = (beatData[tagLen + 11] << 8)
                            + beatData[tagLen + 12];

                    mDeviceInfoTemp.setImageIndex(imgIndex).setImageSpace(imageSpace).setImageLen(imageLen);
                    break;
                case TAG_DEV_IMAGE_DOWN_OVER:
                    mDeviceInfoTemp.setImageStatus(beatData[tagLen + 3]);
                    break;
                case TAG_DEV_LBS:
                    int lbsLen = ((beatData[tagLen + 1] << 8) | beatData[tagLen + 2]);
                    int[] lbsBuf = new int[lbsLen];
                    System.arraycopy(beatData, tagLen + 3, lbsBuf, 0, lbsLen);

                    String devLocation = "";
                    for (int i1 : lbsBuf) {
                        devLocation += HexUtil.toHex(i1);
                    }
                    mDeviceInfoTemp.setDevLocation(devLocation);
                    break;
                case TAG_DEV_OPEN_DEVTIME:
                    byte[] openDVTime = new byte[8];
                    openDVTime[0] = (byte) (((beatData[tagLen + 3] >> 4) & 0x0f) + (byte) 0x30);
                    openDVTime[1] = (byte) (((beatData[tagLen + 3]) & 0x0f) + (byte) 0x30);
                    openDVTime[2] = (byte) ':';
                    openDVTime[3] = (byte) (((beatData[tagLen + 4] >> 4) & 0x0f) + (byte) 0x30);
                    openDVTime[4] = (byte) (((beatData[tagLen + 4]) & 0x0f) + (byte) 0x30);
                    openDVTime[5] = (byte) ':';
                    openDVTime[6] = (byte) (((beatData[tagLen + 5] >> 4) & 0x0f) + (byte) 0x30);
                    openDVTime[7] = (byte) (((beatData[tagLen + 5]) & 0x0f) + (byte) 0x30);

                    byte[] stopDVTime = new byte[8];
                    stopDVTime[0] = (byte) (((beatData[tagLen + 6] >> 4) & 0x0f) + (byte) 0x30);
                    stopDVTime[1] = (byte) (((beatData[tagLen + 6]) & 0x0f) + (byte) 0x30);
                    stopDVTime[2] = (byte) ':';
                    stopDVTime[3] = (byte) (((beatData[tagLen + 7] >> 4) & 0x0f) + (byte) 0x30);
                    stopDVTime[4] = (byte) (((beatData[tagLen + 7]) & 0x0f) + (byte) 0x30);
                    stopDVTime[5] = (byte) ':';
                    stopDVTime[6] = (byte) (((beatData[tagLen + 8] >> 4) & 0x0f) + (byte) 0x30);
                    stopDVTime[7] = (byte) (((beatData[tagLen + 8]) & 0x0f) + (byte) 0x30);

                    mDeviceInfoTemp.setOpenDvTime(openDVTime);
                    mDeviceInfoTemp.setStopDvTime(stopDVTime);
                    break;
                case TAG_DEV_SE_ACTIVATION_UP:
                    log.info("进入了激活");
                    SeActivating seActivating = new SeActivating();
                    int i = tagLen + 3;
                    // nonce se随机数
                    int[] nonceTemp = new int[McuActivationConstant.NONCE_LENGTH];
                    System.arraycopy(beatData, i, nonceTemp, 0, nonceTemp.length);
                    byte[] nonce = DataTransitionUtil.intsToBytes(nonceTemp);
                    seActivating.setNonce(Convert.toHex(nonce));
                    i += nonce.length;
                    // se_id
                    int[] seIdTemp = new int[McuActivationConstant.SE_ID_LENGTH];
                    System.arraycopy(beatData, i, seIdTemp, 0, seIdTemp.length);
                    byte[] seId = DataTransitionUtil.intsToBytes(seIdTemp);
                    seActivating.setSeId(Convert.toHex(seId));
                    i += seId.length;
                    // ts
                    int[] tsTemp = new int[McuActivationConstant.TS_LENGTH];
                    System.arraycopy(beatData, i, tsTemp, 0, tsTemp.length);
                    byte[] ts = DataTransitionUtil.intsToBytes(tsTemp);
                    seActivating.setTimes(Convert.toHex(ts));
                    i += ts.length;
                    // sign
                    int[] signTemp = new int[McuActivationConstant.SIGN_LENGTH];
                    System.arraycopy(beatData, i, signTemp, 0, signTemp.length);
                    byte[] sign = DataTransitionUtil.intsToBytes(signTemp);
                    seActivating.setSignedData(Convert.toHex(sign));
                    i += sign.length;
                    // seCer
                    int[] seCerTemp = new int[McuActivationConstant.SE_CER_LENGTH];
                    System.arraycopy(beatData, i, seCerTemp, 0, seCerTemp.length);
                    byte[] seCer = DataTransitionUtil.intsToBytes(seCerTemp);
                    seActivating.setSeCer(Convert.toHex(seCer));
                    mDeviceInfoTemp.setSeActivating(seActivating);
                    mDeviceInfoTemp.setSeActivatingTag(TAG_DEV_SE_ACTIVATION_UP);
                    break;
                case TAG_DEV_SE_UP:
                    log.info("进入了请求秘钥");
                    mDeviceInfoTemp.setSeActivatingTag(TAG_DEV_SE_UP);
                    break;
                default:break;
            }

        }
        return mDeviceInfoTemp;
    }
}
